'''
Copyright 2017 Recruit Institute of Technology

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
'''

import os

class DictionaryEntityClassifier:
    def __init__(self, filename):
        self.dictionary = {}
        self.load_dictionary(filename)

    def load_dictionary(self, filename):
        if not filename:
            return
        if not os.path.exists(filename):
            print('File not found: %s' % filename)
            return

        file = open(filename)
        for line in file.readlines():
            self.dictionary[line.rstrip()] = 1
        file.close()

    def is_entity(self, name):
        if name in self.dictionary:
            return self.dictionary[name]
        else:
            return 0.0
